#!/bin/sh

# Copyright (c) 2006 Red Hat, Inc. All rights reserved. This copyrighted material 
# is made available to anyone wishing to use, modify, copy, or
# redistribute it subject to the terms and conditions of the GNU General
# Public License v.2.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
# PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# Author: Jan Tluka <jtluka@redhat.com>

. /usr/bin/rhts-environment.sh

prepare() {
	service nfs stop
	[ -f /etc/exports ] && mv /etc/exports /etc/exports.436092
	echo "/tmp *(rw,fsid=0,insecure,no_subtree_check,no_root_squash)" > /etc/exports
	service nfs start
	exportfs | grep "/tmp" 
	[ $? -ne 0 ] && echo "NFS filesystem not exported!" && return 1
	if [ ! -d /mnt/testarea/nfs_436092 ]; then 
		mkdir /mnt/testarea/nfs_436092
	fi
	mount -t nfs4 -o soft localhost:/ /mnt/testarea/nfs_436092
	mount 
	grep /mnt/testarea/nfs_436092 /etc/mtab
	[ $? -ne 0 ] && echo "Failed to mount NFS filesystem" && return 1
	cat ./runtest.sh > /tmp/testfile_436092
}

cleanup() {
	service rpcidmapd stop
	sleep 1

	while true; do 
		pgrep -f reproducer > /dev/null
		if [ $? -eq 1 ]; then 
			break
		fi
	done

	grep /mnt/testarea/nfs_436092 /etc/mtab && umount /mnt/testarea/nfs_436092
	rm -rf /mnt/testarea/nfs_436092
	[ -f /etc/exports.436092 ] && mv /etc/exports.436092 /etc/exports
}

check_bug() {
	./reproducer /mnt/testarea/nfs_436092/testfile_436092 &

	sleep 1
	service nfs stop

	sleep 1

	repro_pid=`jobs -p`
	echo "Wait one minute for a panic ..."
	kill $repro_pid

	# wait 2 minutes (1 minute should be enough)
#	sleep $(( 2 * 60 ))

	echo "If you see this line your system has the bug fixed."

	return 0
}

{
	echo "--- Starting test ..."
	echo "Running on kernel:"
	uname -a
	echo ""

	echo "--- Setting up the environment ..."
	prepare
	echo ""

	echo "--- Checking bug presence ..."
	check_bug
	RC=$?
	echo ""

	echo "--- Cleaning up the system ..."
	cleanup
	echo ""

	echo -n "--- Finished testing: "
	if [ $RC -eq 0 ]; then 
		echo "PASS"
	else
		echo "FAILED"
	fi

	report_result ${TEST} ${RC}

} 2>&1 | tee ${OUTPUTFILE}

